@@ -22,9 +22,11 @@ from fhir.resources.patient import Patient
 from fhir.resources.person import Person
 from phonenumbers import PhoneNumber
 
+from helix_personmatching.fhir_manager.parse_name_result import ParseNameResult
 from helix_personmatching.logics.scoring_input import ScoringInput
 
 from scourgify import normalize_address_record
+from nominally import parse_name
 
 
 class FhirToAttributeDict:
@@ -54,7 +56,7 @@ class FhirToAttributeDict:
                 if address.postalCode and len(address.postalCode) >= 5
                 else None,
             }
-            if address
+            if address and address.postalCode and address.city and address.state
             else None
         )
 
@@ -66,45 +68,69 @@ class FhirToAttributeDict:
             else None
         )
 
-        # parsed_address, parsed_address_type = FhirToAttributeDict.safe_tag_address(
-        #     combined_address,
-        #     tag_mapping={
-        #         'Recipient': 'recipient',
-        #         'AddressNumber': 'address_line_1',
-        #         'AddressNumberPrefix': 'address_line_1',
-        #         'AddressNumberSuffix': 'address_line_1',
-        #         'StreetName': 'address_line_1',
-        #         'StreetNamePreDirectional': 'address_line_1',
-        #         'StreetNamePreModifier': 'address_line_1',
-        #         'StreetNamePreType': 'address_line_1',
-        #         'StreetNamePostDirectional': 'address_line_1',
-        #         'StreetNamePostModifier': 'address_line_1',
-        #         'StreetNamePostType': 'address_line_1',
-        #         'CornerOf': 'address_line_1',
-        #         'IntersectionSeparator': 'address_line_1',
-        #         'LandmarkName': 'address_line_1',
-        #         'USPSBoxGroupID': 'address_line_1',
-        #         'USPSBoxGroupType': 'address_line_1',
-        #         'USPSBoxID': 'address_line_1',
-        #         'USPSBoxType': 'address_line_1',
-        #         'BuildingName': 'address_line_2',
-        #         'OccupancyType': 'address_line_2',
-        #         'OccupancyIdentifier': 'address_line_2',
-        #         'SubaddressIdentifier': 'address_line_2',
-        #         'SubaddressType': 'address_line_2',
-        #         'PlaceName': 'city',
-        #         'StateName': 'state',
-        #         'ZipCode': 'postal_code',
-        #     }
-        # )
-        #
+        parsed_address: Optional[OrderedDict[str, Union[List[str], str]]]
+        parsed_address_type: Optional[str]
+        parsed_address, parsed_address_type = (
+            FhirToAttributeDict.safe_tag_address(
+                combined_address,
+                tag_mapping={
+                    "Recipient": "recipient",
+                    "AddressNumber": "address_line_1",
+                    "AddressNumberPrefix": "address_line_1",
+                    "AddressNumberSuffix": "address_line_1",
+                    "StreetName": "address_line_1",
+                    "StreetNamePreDirectional": "address_line_1",
+                    "StreetNamePreModifier": "address_line_1",
+                    "StreetNamePreType": "address_line_1",
+                    "StreetNamePostDirectional": "address_line_1",
+                    "StreetNamePostModifier": "address_line_1",
+                    "StreetNamePostType": "address_line_1",
+                    "CornerOf": "address_line_1",
+                    "IntersectionSeparator": "address_line_1",
+                    "LandmarkName": "address_line_1",
+                    "USPSBoxGroupID": "address_line_1",
+                    "USPSBoxGroupType": "address_line_1",
+                    "USPSBoxID": "address_line_1",
+                    "USPSBoxType": "address_line_1",
+                    "BuildingName": "address_line_2",
+                    "OccupancyType": "address_line_2",
+                    "OccupancyIdentifier": "address_line_2",
+                    "SubaddressIdentifier": "address_line_2",
+                    "SubaddressType": "address_line_2",
+                    "PlaceName": "city",
+                    "StateName": "state",
+                    "ZipCode": "postal_code",
+                },
+            )
+            if combined_address
+            else (None, None)
+        )
+        # print(parsed_address_type)
+
         address_formatted: Optional[Mapping[str, str]] = (
             FhirToAttributeDict.safe_normalize_address_record(address_dict)
             if address_dict
+            # normalization fails on PO Boxes
+            and (
+                not parsed_address_type
+                or parsed_address_type not in ["PO Box", "Ambiguous"]
+            )
+            and address_dict.get("postal_code")
+            and address_dict.get("city")
+            and address_dict.get("state")
+            else FhirToAttributeDict.safe_normalize_address_record(parsed_address)  # type: ignore
+            if parsed_address
+            # normalization fails on PO Boxes
+            and (
+                not parsed_address_type
+                or parsed_address_type not in ["PO Box", "Ambiguous"]
+            )
             else None
         )
 
         # https://github.com/datamade/usaddress
+        address_tagged: Optional[OrderedDict[str, Union[List[Any], str]]]
+        address_type: Optional[str]
         # noinspection PyUnresolvedReferences
         address_tagged, address_type = (
             (FhirToAttributeDict.safe_tag_address(combined_address))
@@ -139,41 +165,90 @@ class FhirToAttributeDict:
 
         age_in_years = FhirToAttributeDict.calculate_age_in_years(resource.birthDate)
 
+        # Get postal code in any way we can
+        address_postal_code: Optional[str] = None
+        if address_formatted and "postal_code" in address_formatted:
+            address_postal_code = address_formatted.get("postal_code")
+        if not address_postal_code and address_tagged and "ZipCode" in address_tagged:
+            address_tagged_zip_code = address_tagged.get("ZipCode")
+            if (
+                isinstance(address_tagged_zip_code, list)
+                and len(address_tagged_zip_code) > 0
+            ):
+                address_postal_code = address_tagged_zip_code[0]
+            elif not isinstance(address_tagged_zip_code, list):
+                address_postal_code = address_tagged_zip_code
+        if (
+            not address_postal_code
+            and parsed_address
+            and "postal_code" in parsed_address
+        ):
+            parsed_address_zip_code = parsed_address.get("postal_code")
+            if (
+                isinstance(parsed_address_zip_code, list)
+                and len(parsed_address_zip_code) > 0
+            ):
+                address_postal_code = parsed_address_zip_code[0]
+            elif not isinstance(parsed_address_zip_code, list):
+                address_postal_code = parsed_address_zip_code
         # noinspection PyUnresolvedReferences
-        address_postal_code = (
-            address_formatted.get("postal_code")
-            if address_formatted
-            else address.postalCode
-            if address and address.postalCode
-            else None
-        )
+        if not address_postal_code and address and address.postalCode:
+            # noinspection PyUnresolvedReferences
+            address_postal_code = address.postalCode
+
+        # Get address line 1 any way we can
+        address_line_1: Optional[str] = None
+        if address_formatted and "address_line_1" in address_formatted:
+            address_line_1 = address_formatted.get("address_line_1")
+        if not address_line_1 and parsed_address and "address_line_1" in parsed_address:
+            parsed_address_line_1 = parsed_address.get("address_line_1")
+            if (
+                isinstance(parsed_address_line_1, list)
+                and len(parsed_address_line_1) > 0
+            ):
+                address_line_1 = parsed_address_line_1[0]
+            elif not isinstance(parsed_address_line_1, list):
+                address_line_1 = parsed_address_line_1
+        # noinspection PyUnresolvedReferences
+        if not address_line_1 and address and address.line and len(address.line) > 0:
+            # noinspection PyUnresolvedReferences
+            address_line_1 = address.line[0]
 
         # noinspection PyUnresolvedReferences
-        address_line_1 = (
-            address_formatted.get("address_line_1")
-            if address_formatted
-            else address.line[0]
-            if address and address.line and len(address.line) > 0
+        first_name = (
+            patient_name.given[0]
+            if patient_name and patient_name.given and len(patient_name.given) > 0
             else None
         )
-
         # noinspection PyUnresolvedReferences
-        name_middle = (
+        family_name = patient_name.family if patient_name else None
+        # noinspection PyUnresolvedReferences
+        middle_name = (
             patient_name.given[1]
             if patient_name and patient_name.given and len(patient_name.given) > 1
             else None
         )
 
+        # try to parse names using nominally since the names can be stored in wrong fields
+        parsed_name: Optional[ParseNameResult] = FhirToAttributeDict.safe_name_parse(
+            name=patient_name
+        )
+        if parsed_name is not None:
+            if parsed_name.first:
+                first_name = parsed_name.first
+            if parsed_name.middle:
+                middle_name = parsed_name.middle
+            if parsed_name.last:
+                family_name = parsed_name.last
+
         # noinspection PyUnresolvedReferences
         scoring_input: ScoringInput = ScoringInput(
             id_=resource.id,
-            name_given=patient_name.given[0]
-            if patient_name and patient_name.given and len(patient_name.given) > 0
-            else None,
-            name_family=patient_name.family if patient_name else None,
-            name_middle=name_middle,
-            name_middle_initial=name_middle[0]
-            if name_middle and len(name_middle) > 0
+            name_given=first_name,
+            name_family=family_name,
+            name_middle=middle_name,
+            name_middle_initial=middle_name[0]
+            if middle_name and len(middle_name) > 0
             else None,
             gender=resource.gender,
             birth_date=resource.birthDate.strftime("%Y-%m-%d")
@@ -211,6 +286,12 @@ class FhirToAttributeDict:
     def safe_normalize_address_record(
         address: Union[str, Mapping[str, str]]
     ) -> Mapping[str, str]:
+        if not isinstance(address, str) and (
+            not address.get("postal_code")
+            or not address.get("city")
+            or not address.get("state")
+        ):
+            return address
         try:
             return cast(Mapping[str, str], normalize_address_record(address=address))
         # except UnParseableAddressError:
@@ -235,10 +316,13 @@ class FhirToAttributeDict:
     @staticmethod
     def safe_tag_address(
         address_line: Optional[str], tag_mapping: Optional[Mapping[str, str]] = None
-    ) -> Tuple[OrderedDict[str, Union[List[Any], str]], str]:
+    ) -> Tuple[OrderedDict[str, Union[List[str], str]], str]:
+        if not address_line:
+            tagged_address = OrderedDict[str, Union[List[Any], str]]()
+            return tagged_address, "Ambiguous"
         try:
             return cast(
-                Tuple[OrderedDict[str, Union[List[Any], str]], str],
+                Tuple[OrderedDict[str, Union[List[str], str]], str],
                 usaddress.tag(address_line, tag_mapping=tag_mapping),
             )
         except Exception as e:
@@ -375,3 +459,28 @@ class FhirToAttributeDict:
         age = year_difference - one_or_zero
 
         return age
+
+    @staticmethod
+    def safe_name_parse(name: Optional[HumanNameType]) -> Optional[ParseNameResult]:
+        # noinspection PyUnresolvedReferences
+        if name is None or name.family is None:
+            return None
+        combined_name = ""
+        try:
+            # noinspection PyUnresolvedReferences
+            if name.given is not None:
+                # noinspection PyUnresolvedReferences
+                combined_name += " ".join(name.given)
+            # noinspection PyUnresolvedReferences
+            if name.family is not None:
+                # noinspection PyUnresolvedReferences
+                combined_name += f" {name.family}"
+            result = parse_name(combined_name)
+            return ParseNameResult(
+                first=result.get("first"),
+                middle=result.get("middle"),
+                last=result.get("last"),
+            )
+        except Exception as e:
+            print(f"Name Parsing Error: Parsing Name: {combined_name}: {e}")
+            return None
