@@ -46,6 +46,7 @@ class ScoreCalculator:
                 id_target=target.id_,
                 rule_scores=match_results,
                 total_score=0.0,
+                total_score_unscaled=0.0,
                 average_boost=None,
                 average_score=0.0,
             )
@@ -74,25 +75,14 @@ class ScoreCalculator:
         if average_score_boost is not None:
             score_average = sum_of_scores / rule_count
             score_average_boost = score_average * average_score_boost
-            if final_score + score_average_boost >= 1.0:
-                # scale the boost, so we don't go over 1.0
-                diff = 1.0 - final_score
-                max_average_score_boost = 1 * average_score_boost
-                score_average_boost = (
-                    score_average
-                    * (average_score_boost / max_average_score_boost)
-                    * diff
-                )
             final_score += score_average_boost
 
-        # cap total score at 1.0
-        final_score = 1.0 if final_score > 1.0 else final_score
-
         return MatchScoreWithoutThreshold(
             id_source=source.id_,
             id_target=target.id_,
             rule_scores=match_results,
             total_score=final_score,
+            total_score_unscaled=final_score,
             average_boost=score_average_boost,
             average_score=score_average,
         )
