@@ -1,4 +1,3 @@
-import dataclasses
 import json
 from pathlib import Path
 from typing import List
@@ -34,12 +33,10 @@ def test_walgreens_partial_match() -> None:
     print(f"score.total_score: {score.total_score}")
     assert score.total_score > 0.995
 
-    score_dict = dataclasses.asdict(score)
-
-    scores_json = json.dumps(score_dict, default=str)
+    scores_json = score.to_json()
     print(scores_json)
 
     with open(data_dir.joinpath("expected_scores.json")) as file:
         expected_scores = json.loads(file.read())
 
-    assert score_dict == expected_scores
+    assert json.loads(scores_json) == expected_scores
