@@ -1,5 +1,6 @@
-from typing import List, Optional, Any
+from typing import List, Optional, Any, Set, Tuple
 
+from nicknames import NickNamer
 from rapidfuzz import distance
 
 from helix_personmatching.logics.rule_attribute_score import RuleAttributeScore
@@ -7,7 +8,9 @@ from helix_personmatching.logics.rule_score import RuleScore
 from helix_personmatching.logics.scoring_input import ScoringInput
 from helix_personmatching.models.attribute_entry import AttributeEntry
 from helix_personmatching.models.rule import Rule
+from helix_personmatching.models.rule_option import RuleOption
 from helix_personmatching.models.rules.RuleWeight import RuleWeight
+from helix_personmatching.models.string_match_type import StringMatchType
 
 
 class AttributeRule(Rule):
@@ -25,11 +28,14 @@ class AttributeRule(Rule):
         )
         self.attributes: List[AttributeEntry] = attributes
 
-    def score(self, source: ScoringInput, target: ScoringInput) -> Optional[RuleScore]:
+    def score(
+        self, source: ScoringInput, target: ScoringInput, rule_option: RuleOption
+    ) -> Optional[RuleScore]:
         """
         Calculate a matching score for one rule between FHIR Person-Person, or Person-Patient, or Person/Patient-AppUser
 
 
+        :param rule_option:
         :param source: Dictionary of Pii data for FHIR Person/Patient data, or AppUser data
         :param target: Dictionary of Pii data for FHIR Person/Patient data, or AppUser data
         :return: Dictionary of 1 rule score result
@@ -54,14 +60,18 @@ class AttributeRule(Rule):
 
                 # calculate exact string match on "trimmed lower" string values
                 # returns a number between 0 and 1
-                score_for_attribute = AttributeRule.calculate_string_match(
-                    attribute, val_source, val_target
+                score_for_attribute, string_match_type = self.calculate_string_match(
+                    attribute=attribute,
+                    val_source=val_source,
+                    val_target=val_target,
+                    nick_namer=rule_option.nick_namer,
                 )
                 score_avg += score_for_attribute
 
                 rule_attribute_score.score = score_for_attribute
                 rule_attribute_score.source = val_source
                 rule_attribute_score.target = val_target
+                rule_attribute_score.string_match_type = string_match_type
             elif not val_source or not val_target:
                 rule_attribute_score.score = self.weight.missing
                 score_avg += self.weight.missing
@@ -91,8 +101,12 @@ class AttributeRule(Rule):
 
     @staticmethod
     def calculate_string_match(
-        attribute: AttributeEntry, val_source: Optional[str], val_target: Optional[str]
-    ) -> float:
+        *,
+        attribute: AttributeEntry,
+        val_source: Optional[str],
+        val_target: Optional[str],
+        nick_namer: NickNamer
+    ) -> Tuple[float, StringMatchType]:
         """
         Returns a score from 0 to 1 where 1 is an exact match
 
@@ -100,6 +114,7 @@ class AttributeRule(Rule):
         :param attribute:
         :param val_source:
         :param val_target:
+        :param nick_namer:
         :return:
         """
         val_source_cleansed: str = str(val_source).strip().lower()
@@ -107,9 +122,20 @@ class AttributeRule(Rule):
 
         if attribute.exact_only:
             if val_source_cleansed == val_target_cleansed:
-                return 1.0
+                return 1.0, StringMatchType.Exact
             else:
-                return 0.0
+                return 0.0, StringMatchType.NoMatch
+
+        if attribute.nick_name_match:
+            canonicals_source: Set[str] = nick_namer.canonicals_of(
+                val_source_cleansed
+            ) | {val_source_cleansed}
+            canonicals_target: Set[str] = nick_namer.canonicals_of(
+                val_target_cleansed
+            ) | {val_target_cleansed}
+            matching_set: Set[str] = canonicals_source.intersection(canonicals_target)
+            if len(matching_set) > 0:
+                return 1.0, StringMatchType.Synonym
 
         score_for_attribute = distance.Levenshtein.normalized_similarity(
             val_source_cleansed,
@@ -119,24 +145,9 @@ class AttributeRule(Rule):
             score_cutoff=0.7,  # usually no more than two character changes
         )
 
-        return score_for_attribute
-
-    @staticmethod
-    # In case for Considering ONLY available 'present' attributes in both source and target data.
-    # i.e. Do not penalize the total rule average score by not available attributes.
-    def get_total_number_of_available_attributes(
-        rule_attribute_scores: List[RuleAttributeScore],
-    ) -> int:
-        total_number_of_available_attributes: int = sum(
-            map(
-                lambda x: x.present is True
-                and x.source is not None
-                and x.target is not None,
-                rule_attribute_scores,
-            )
-        )
         return (
-            1
-            if total_number_of_available_attributes == 0
-            else total_number_of_available_attributes
+            score_for_attribute,
+            StringMatchType.Partial
+            if score_for_attribute < 1.0
+            else StringMatchType.Exact,
         )
